@@ -7,7 +7,6 @@
 #include "selectors_p.h"
 #include "logging.h"
 #include "openinghours_p.h"
-#include "consecutiveaccumulator_p.h"
 
 #include <cstdlib>
 #include <cassert>
@@ -153,13 +152,13 @@ bool Timespan::operator==(Timespan &other) const
 
 int WeekdayRange::requiredCapabilities() const
 {
-    // only ranges or nthMask are allowed, not both at the same time, enforced by parser
-    assert(beginDay == endDay || nthMask == 0);
+    // only ranges or nthSequence are allowed, not both at the same time, enforced by parser
+    assert(beginDay == endDay || !nthSequence);
 
     int c = Capability::None;
     switch (holiday) {
         case NoHoliday:
-            if ((offset > 0 && nthMask == 0)) {
+            if ((offset > 0 && !nthSequence)) {
                 c |= Capability::NotImplemented;
             }
             break;
@@ -201,21 +200,8 @@ QByteArray WeekdayRange::toExpression() const
             expr = "SH";
             break;
         }
-        if (nthMask > 0) {
-            ConsecutiveAccumulator accu([](int i) { return QByteArray::number(i); });
-            // Negative numbers
-            for (int i = 1; i <= 10; i += 2) {
-                if ((nthMask & (1 << i)) != 0) {
-                    accu.add(-5 + (i / 2));
-                }
-            }
-            // Positive numbers
-            for (int i = 2; i <= 10; i += 2) {
-                if ((nthMask & (1 << i)) != 0) {
-                    accu.add(i / 2);
-                }
-            }
-            expr += '[' + accu.result() + ']';
+        if (nthSequence) {
+            expr += '[' + nthSequence->toExpression() + ']';
         }
         if (offset > 0) {
             expr += " +" + QByteArray::number(offset) + ' ' + (offset > 1 ? "days" : "day");
@@ -237,7 +223,7 @@ void WeekdayRange::simplify()
     std::fill(std::begin(seenDays), std::end(seenDays), false);
     for (WeekdayRange *selector = this; selector; selector = selector->next.get()) {
         // Ensure it's all just week days, no other features
-        if (selector->nthMask || selector->lhsAndSelector || selector->holiday != NoHoliday || selector->offset) {
+        if (selector->nthSequence || selector->lhsAndSelector || selector->holiday != NoHoliday || selector->offset) {
             return;
         }
         const bool wrap = selector->beginDay > selector->endDay;
@@ -352,7 +338,7 @@ QByteArray Week::toExpression() const
     return expr;
 }
 
-QByteArray Date::toExpression(Date refDate) const
+QByteArray Date::toExpression(const Date &refDate, const MonthdayRange &prev) const
 {
     QByteArray expr;
     auto maybeSpace = [&]() {
@@ -361,20 +347,26 @@ QByteArray Date::toExpression(Date refDate) const
         }
     };
     switch (variableDate) {
-    case FixedDate:
-        if (year && year != refDate.year) {
+    case FixedDate: {
+        const bool needYear = year && year != refDate.year;
+        if (needYear) {
             expr += QByteArray::number(year);
         }
-        if (month && ((year && year != refDate.year) || month != refDate.month || hasOffset())) {
-            static const char* s_monthName[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
-            maybeSpace();
-            expr += s_monthName[month-1];
+        if (month) {
+            const bool combineWithPrev = prev.begin.month == prev.end.month && month == prev.begin.month;
+            const bool implicitMonth = month == refDate.month || (refDate.month == 0 && combineWithPrev);
+            if (needYear || !implicitMonth || hasOffset()) {
+                static const char* s_monthName[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
+                maybeSpace();
+                expr += s_monthName[month-1];
+            }
         }
         if (day && *this != refDate) {
             maybeSpace();
             expr += twoDigits(day);
         }
         break;
+    }
     case Date::Easter:
         if (year) {
             expr += QByteArray::number(year) + ' ';
@@ -436,18 +428,36 @@ int MonthdayRange::requiredCapabilities() const
     return Capability::None;
 }
 
-QByteArray MonthdayRange::toExpression() const
+QByteArray MonthdayRange::toExpression(const MonthdayRange &prev) const
 {
-    QByteArray expr = begin.toExpression({});
+    QByteArray expr = begin.toExpression({}, prev);
     if (end != begin) {
-        expr += '-' + end.toExpression(begin);
+        expr += '-' + end.toExpression(begin, prev);
     }
     if (next) {
-        expr += ',' + next->toExpression();
+        expr += ',' + next->toExpression(*this);
     }
     return expr;
 }
 
+void MonthdayRange::simplify()
+{
+    // "Feb 1-29" => "Feb" (#446252)
+    if (begin.variableDate == Date::FixedDate &&
+            end.variableDate == Date::FixedDate &&
+            begin.year == end.year &&
+            begin.month && end.month  &&
+            begin.month == end.month  &&
+            begin.day && end.day) {
+        // The year doesn't matter, but take one with a leap day, for Feb 1-29
+         const int lastDay = QDate{2004, end.month, end.day}.daysInMonth();
+         if (begin.day == 1 && end.day == lastDay) {
+             begin.day = 0;
+             end.day = 0;
+         }
+    }
+}
+
 int YearRange::requiredCapabilities() const
 {
     return Capability::None;
@@ -471,3 +481,26 @@ QByteArray YearRange::toExpression() const
     }
     return expr;
 }
+
+void NthSequence::add(NthEntry range)
+{
+    sequence.push_back(std::move(range));
+}
+
+QByteArray NthSequence::toExpression() const
+{
+    QByteArray ret;
+    for (const NthEntry &entry : sequence) {
+        if (!ret.isEmpty())
+            ret += ',';
+        ret += entry.toExpression();
+    }
+    return ret;
+}
+
+QByteArray NthEntry::toExpression() const
+{
+    if (begin == end)
+        return QByteArray::number(begin);
+    return QByteArray::number(begin) + '-' + QByteArray::number(end);
+}
